@@ -31,16 +31,10 @@
 #include <opm/grid/utility/platform_dependent/reenable_warnings.h>
 #endif
 
-#include <dune/common/version.hh>
-#if DUNE_VERSION_NEWER(DUNE_COMMON, 2, 7)
 #include <dune/common/parallel/communication.hh>
-#else
-#include <dune/common/parallel/mpicollectivecommunication.hh>
-#endif
 
-
-#include <opm/grid/utility/OpmParserIncludes.hpp>
 #include <opm/grid/CpGrid.hpp>
+#include <opm/grid/utility/OpmWellType.hpp>
 
 namespace Dune
 {
@@ -163,13 +157,10 @@ postProcessPartitioningForWells(std::vector<int>& parts,
                                 std::function<int(int)> gid,
                                 const std::vector<OpmWellType>&  wells,
                                 const WellConnections& well_connections,
+                                const std::vector<std::set<int> >& wellGraph,
                                 std::vector<std::tuple<int,int,char>>& exportList,
                                 std::vector<std::tuple<int,int,char,int>>& importList,
-#if DUNE_VERSION_NEWER(DUNE_GRID, 2, 7)
                                 const Communication<MPI_Comm>& cc);
-#else
-                                const CollectiveCommunication<MPI_Comm>& cc);
-#endif
 
 /// \brief Computes whether wells are perforating cells on this process.
 /// \param wells_on_proc well indices assigned to each process
@@ -182,13 +173,10 @@ postProcessPartitioningForWells(std::vector<int>& parts,
 std::vector<std::pair<std::string,bool>>
 computeParallelWells(const std::vector<std::vector<int> >& wells_on_proc,
                      const std::vector<OpmWellType>&  wells,
-#if DUNE_VERSION_NEWER(DUNE_GRID, 2, 7)
                      const Communication<MPI_Comm>& cc,
-#else
-                     const CollectiveCommunication<MPI_Comm>& cc,
-#endif
                      int root);
 #endif
 } // end namespace cpgrid
 } // end namespace Dune
+
 #endif
