@@ -1,12 +1,13 @@
 //===========================================================================
 //
-// File: CpGrid.hpp
+// File: CpGridData.hpp
 //
 // Created: Sep 17 21:11:41 2013
 //
 // Author(s): Atgeirr F Rasmussen <atgeirr@sintef.no>
 //            Bård Skaflestad     <bard.skaflestad@sintef.no>
 //            Markus Blatt        <markus@dr-blatt.de>
+//            Antonella Ritorto   <antonella.ritorto@opm-op.com>
 //
 // Comment: Major parts of this file originated in dune/grid/CpGrid.hpp
 //          and got transfered here during refactoring for the parallelization.
@@ -19,7 +20,7 @@
 
 /*
   Copyright 2009, 2010 SINTEF ICT, Applied Mathematics.
-  Copyright 2009, 2010, 2013, 2022 Equinor ASA.
+  Copyright 2009, 2010, 2013, 2022-2023 Equinor ASA.
   Copyright 2013 Dr. Blatt - HPC-Simulation-Software & Services
 
   This file is part of The Open Porous Media project  (OPM).
@@ -47,50 +48,32 @@
 #ifndef OPM_CPGRIDDATA_HEADER
 #define OPM_CPGRIDDATA_HEADER
 
-// Warning suppression for Dune includes.
-#include <opm/grid/utility/platform_dependent/disable_warnings.h>
 
-#include <dune/common/version.hh>
 #include <dune/common/parallel/mpihelper.hh>
 #ifdef HAVE_DUNE_ISTL
 #include <dune/istl/owneroverlapcopy.hh>
 #endif
 
-#if DUNE_VERSION_NEWER(DUNE_COMMON, 2, 7)
 #include <dune/common/parallel/communication.hh>
-#else
-#include <dune/common/parallel/collectivecommunication.hh>
-#endif
-#include <dune/common/parallel/indexset.hh>
-#include <dune/common/parallel/interface.hh>
-#include <dune/common/parallel/plocalindex.hh>
-#if DUNE_VERSION_NEWER(DUNE_GRID, 2, 7)
 #include <dune/common/parallel/variablesizecommunicator.hh>
-#else
-#include <opm/grid/utility/VariableSizeCommunicator.hpp>
-#endif
 #include <dune/grid/common/gridenums.hh>
 
-#include <opm/grid/utility/platform_dependent/reenable_warnings.h>
-
 #if HAVE_ECL_INPUT
 #include <opm/input/eclipse/EclipseState/Grid/NNC.hpp>
 #endif
 
-#include <array>
-#include <tuple>
-#include <algorithm>
-#include <set>
-
-#include "OrientedEntityTable.hpp"
-#include "DefaultGeometryPolicy.hpp"
 #include <opm/grid/cpgpreprocess/preprocess.h>
 
-#include <opm/grid/utility/OpmParserIncludes.hpp>
-
 #include "Entity2IndexDataHandle.hpp"
-#include "DataHandleWrappers.hpp"
-#include "GlobalIdMapping.hpp"
+#include "CpGridDataTraits.hpp"
+//#include "DataHandleWrappers.hpp"
+//#include "GlobalIdMapping.hpp"
+#include "Geometry.hpp"
+
+#include <array>
+#include <set>
+#include <vector>
+
 namespace Opm
 {
 class EclipseState;
@@ -109,7 +92,32 @@ class PartitionTypeIndicator;
 template<int,int> class Geometry;
 template<int> class Entity;
 template<int> class EntityRep;
+}
+}
 
+void refine_and_check(const Dune::cpgrid::Geometry<3, 3>&,
+                      const std::array<int, 3>&,
+                      bool);
+
+void refinePatch_and_check(const std::array<int,3>&,
+                           const std::array<int,3>&,
+                           const std::array<int,3>&);
+
+void refinePatch_and_check(Dune::CpGrid&,
+                           const std::vector<std::array<int,3>>&,
+                           const std::vector<std::array<int,3>>&,
+                           const std::vector<std::array<int,3>>&,
+                           const std::vector<std::string>&);
+
+void check_global_refine(const Dune::CpGrid&,
+                         const Dune::CpGrid&);
+
+void fieldProp_check(const Dune::CpGrid& grid, Opm::EclipseGrid eclGrid, std::string deck_string);
+
+namespace Dune
+{
+namespace cpgrid
+{
 namespace mover
 {
 template<class T, int i> struct Mover;
@@ -122,8 +130,32 @@ template<class T, int i> struct Mover;
 class CpGridData
 {
     template<class T, int i> friend struct mover::Mover;
-
     friend class GlobalIdSet;
+    friend class HierarchicIterator;
+    friend class Dune::cpgrid::IndexSet;
+
+    friend
+    void ::refine_and_check(const Dune::cpgrid::Geometry<3, 3>&,
+                            const std::array<int, 3>&,
+                            bool);
+    friend
+    void ::refinePatch_and_check(const std::array<int,3>&,
+                        const std::array<int,3>&,
+                        const std::array<int,3>&);
+
+    friend
+    void ::refinePatch_and_check(Dune::CpGrid&,
+                                 const std::vector<std::array<int,3>>&,
+                                 const std::vector<std::array<int,3>>&,
+                                 const std::vector<std::array<int,3>>&,
+                                 const std::vector<std::string>&);
+    
+    friend
+    void ::check_global_refine(const Dune::CpGrid&,
+                               const Dune::CpGrid&);
+
+    friend
+    void ::fieldProp_check(const Dune::CpGrid& grid, Opm::EclipseGrid eclGrid, std::string deck_string);
 
 private:
     CpGridData(const CpGridData& g);
@@ -145,17 +177,23 @@ public:
         /// communicating. Uses the define MAX_DATA_COMMUNICATED_PER_ENTITY.
         MAX_DATA_PER_CELL = MAX_DATA_COMMUNICATED_PER_ENTITY
 #endif
-    };
+    };     
 
     /// Constructor for parallel grid data.
     /// \param comm The MPI communicator
     /// Default constructor.
-    explicit CpGridData(MPIHelper::MPICommunicator comm);
+    explicit CpGridData(MPIHelper::MPICommunicator comm,  std::vector<std::shared_ptr<CpGridData>>& data);
 
+
+    
     /// Constructor
-    CpGridData();
+    explicit CpGridData(std::vector<std::shared_ptr<CpGridData>>& data);  
     /// Destructor
     ~CpGridData();
+  
+   
+    
+    
     /// number of leaf entities per codim in this process
     int size(int codim) const;
 
@@ -208,12 +246,15 @@ public:
     ///        side. That is, i- faces will match i+ faces etc.
     /// \param turn_normals if true, all normals will be turned. This is intended for handling inputs with wrong orientations.
     /// \param clip_z if true, the grid will be clipped so that the top and bottom will be planar.
+    /// \param pichActive Whether PINCH keyword was specified
     std::vector<std::size_t> processEclipseFormat(const Opm::EclipseGrid* ecl_grid, Opm::EclipseState* ecl_state,
-                                                  bool periodic_extension, bool turn_normals = false, bool clip_z = false, bool pinchActive = true);
+                                                  bool periodic_extension, bool turn_normals = false, bool clip_z = false,
+                                                  bool pinchActive = true);
 #endif
 
     /// Read the Eclipse grid format ('grdecl').
     /// \param input_data the data in grdecl format, declared in preprocess.h.
+    ///
     /// \param ecl_state the object from opm-parser provide information regarding to pore volume, NNC,
     ///        aquifer information when ecl_state is available. NNC and aquifer connection
     ///        information will also be updated during the function call when available and necessary.
@@ -221,9 +262,14 @@ public:
     /// \param remove_ij_boundary if true, will remove (i, j) boundaries. Used internally.
     /// \param pinchActive If true, we will add faces between vertical cells that have only inactive cells or cells
     ///            with zero volume between them. If false these cells will not be connected.
-    void processEclipseFormat(const grdecl& input_data, Opm::EclipseState* ecl_state,
+    /// \param tolerance_unique_points Tolerance used to identify points based on their cooridinate
+    void processEclipseFormat(const grdecl& input_data,
+#if HAVE_ECL_INPUT
+                              Opm::EclipseState* ecl_state,
+#endif
                               std::array<std::set<std::pair<int, int>>, 2>& nnc,
-                              bool remove_ij_boundary, bool turn_normals, bool pinchActive);
+                              bool remove_ij_boundary, bool turn_normals, bool pinchActive,
+                              double tolerance_unique_points);
 
     /// @brief
     ///    Extract Cartesian index triplet (i,j,k) of an active cell.
@@ -240,6 +286,195 @@ public:
         ijk[2] = gc / logical_cartesian_size_[1];
     }
 
+    /// @brief Determine if a finite amount of patches (of cells) are disjoint, namely, they do not share any corner nor face.
+    ///
+    /// @param [in]  startIJK_vec  Vector of Cartesian triplet indices where each patch starts.
+    /// @param [in]  endIJK_vec    Vector of Cartesian triplet indices where each patch ends.
+    ///                            Last cell part of the lgr will be {endIJK_vec[<patch>][0]-1, ... ,endIJK_vec[<patch>][2]-1}.
+    bool disjointPatches(const std::vector<std::array<int,3>>& startIJK_vec, const std::vector<std::array<int,3>>& endIJK_vec) const;
+
+    /// @brief Compute cell indices of selected patches of cells (Cartesian grid required).
+    ///
+    /// @param [in]  startIJK_vec  Vector of Cartesian triplet indices where each patch starts.
+    /// @param [in]  endIJK_vec    Vector of Cartesian triplet indices where each patch ends.
+    ///                            Last cell part of the lgr will be {endIJK_vec[<patch>][0]-1, ... endIJK_vec[<patch>][2]-1}.
+    ///
+    /// @return allPatches_cells
+    std::vector<int>
+    getPatchesCells(const std::vector<std::array<int,3>>& startIJK_vec, const std::vector<std::array<int,3>>& endIJK_vec) const;
+
+    /// @brief Check all cells selected for refinement have no NNCs (no neighbor connections).
+    ///        Assumption: all grid cells are active.
+    bool hasNNCs(const std::vector<int>& cellIndices) const;
+
+    /// @brief Check startIJK and endIJK of each patch of cells to be refined are valid, i.e.
+    ///        startIJK and endIJK vectors have the same size and, startIJK < endIJK coordenate by coordenate.
+    ///
+    /// @param [in]  startIJK_vec  Vector of Cartesian triplet indices where each patch starts.
+    /// @param [in]  endIJK_vec    Vector of Cartesian triplet indices where each patch ends.
+    ///                            Last cell part of the lgr will be {endIJK_vec[patch][0]-1, ..., endIJK_vec[patch][2]-1}.
+    void validStartEndIJKs(const std::vector<std::array<int,3>>& startIJK_vec, const std::vector<std::array<int,3>>& endIJK_vec) const;
+
+    /// @brief Check that every cell to be refined has cuboid shape.
+    void checkCuboidShape(const std::vector<int>& cellIdx_vec) const;
+
+    /// @brief Determine if a finite amount of patches (of cells) share a face.
+    ///
+    /// @param [in]  startIJK_vec  Vector of Cartesian triplet indices where each patch starts.
+    /// @param [in]  endIJK_vec    Vector of Cartesian triplet indices where each patch ends.
+    ///                            Last cell part of the lgr will be {endIJK_vec[<patch>][0]-1, ... ,endIJK_vec[<patch>][2]-1}.
+    bool patchesShareFace(const std::vector<std::array<int,3>>& startIJK_vec, const std::vector<std::array<int,3>>& endIJK_vec) const;
+
+private:
+    /// @brief Compute amount of cells in each direction of a patch of cells. (Cartesian grid required).
+    ///
+    /// @param [in]  startIJK  Cartesian triplet index where the patch starts.
+    /// @param [in]  endIJK    Cartesian triplet index where the patch ends.
+    ///                        Last cell part of the lgr will be {endijk[0]-1, ... endIJK[2]-1}.
+    ///
+    /// @return patch_dim Patch dimension {#cells in x-direction, #cells in y-direction, #cells in z-direction}.
+    std::array<int,3> getPatchDim(const std::array<int,3>& startIJK, const std::array<int,3>& endIJK) const;
+
+    /// @brief Compute corner indices of a patch of cells (Cartesian grid required).
+    ///
+    /// @param [in]  startIJK  Cartesian triplet index where the patch starts.
+    /// @param [in]  endIJK    Cartesian triplet index where the patch ends.
+    ///                        Last cell part of the lgr will be {endijk[0]-1, ... endIJK[2]-1}.
+    ///
+    /// @return patch_corners
+    std::vector<int> getPatchCorners(const std::array<int,3>& startIJK, const std::array<int,3>& endIJK) const;
+
+    /// @brief Compute face indices of a patch of cells (Cartesian grid required).
+    ///
+    /// @param [in]  startIJK  Cartesian triplet index where the patch starts.
+    /// @param [in]  endIJK    Cartesian triplet index where the patch ends.
+    ///                        Last cell part of the lgr will be {endijk[0]-1, ... endIJK[2]-1}.
+    ///
+    /// @return patch_faces
+    std::vector<int> getPatchFaces(const std::array<int,3>& startIJK, const std::array<int,3>& endIJK) const;
+
+    /// @brief Compute cell indices of a patch of cells (Cartesian grid required).
+    ///
+    /// @param [in]  startIJK  Cartesian triplet index where the patch starts.
+    /// @param [in]  endIJK    Cartesian triplet index where the patch ends.
+    ///                        Last cell part of the lgr will be {endIJK[0]-1, ... endIJK[2]-1}.
+    ///
+    /// @return patch_cells
+    std::vector<int> getPatchCells(const std::array<int,3>& startIJK, const std::array<int,3>& endIJK) const;
+
+    /// @brief Compute patch boundary corner indices (Cartesian grid required).
+    ///
+    /// @param [in]  startIJK  Cartesian triplet index where the patch starts.
+    /// @param [in]  endIJK    Cartesian triplet index where the patch ends.
+    ///                        Last cell part of the lgr will be {endijk[0]-1, ... endIJK[2]-1}.
+    ///
+    /// @return patch_boundary_corners
+    std::vector<int> getPatchBoundaryCorners(const std::array<int,3>& startIJK, const std::array<int,3>& endIJK) const;
+
+    /// @brief Compute patch boundary face indices (Cartesian grid required).
+    ///
+    /// @param [in]  startIJK  Cartesian triplet index where the patch starts.
+    /// @param [in]  endIJK    Cartesian triplet index where the patch ends.
+    ///                        Last cell part of the lgr will be {endijk[0]-1, ... endIJK[2]-1}.
+    ///
+    /// @return patch_boundary_faces
+    std::array<std::vector<int>,6> getBoundaryPatchFaces(const std::array<int,3>& startIJK, const std::array<int,3>& endIJK) const;
+
+    /// @brief For selected cell indices, computes the variation in x-,y-, and z-direction, assuming each cell has cubiod shape.
+    std::array<std::vector<double>,3> getWidthsLengthsHeights(const std::array<int,3>& startIJK, const std::array<int,3>& endIJK) const;
+
+    /// @brief Construct a 'fake cell (Geometry<3,3> object)' out of a patch of cells.(Cartesian grid required).
+    ///
+    /// cellifyPatch() builds a Geometry<3,3> object, 'a celliFIED patch', from a connected patch formed
+    /// by the product of consecutive cells in each direction; selecting 8 corners of the patch boundary,
+    /// computing center and volume.
+    ///
+    /// @param [in] startIJK                   Cartesian triplet index where the patch starts.
+    /// @param [in] endIJK                     Cartesian triplet index where the patch ends.
+    /// @param [in] patch_cells                Cell indices from the block-shaped patch.
+    /// @param [out] cellifiedPatch_geometry   Required as an argument when creating a Geomtry<3,3> object.
+    /// @param [out] cellifiedPatch_to_point   To store the 8 corners of the created cellifiedPatch.
+    /// @param [out] allcorners_cellifiedPatch Required to build a Geometry<3,3> object.
+    ///
+    /// @return 'cellifiedPatchCell'         Geometry<3,3> object.
+    Geometry<3,3> cellifyPatch(const std::array<int,3>& startIJK, const std::array<int,3>& endIJK,
+                               const std::vector<int>& patch_cells, DefaultGeometryPolicy& cellifiedPatch_geometry,
+                               std::array<int,8>& cellifiedPatch_to_point,
+                               std::array<int,8>& allcorners_cellifiedPatch) const;
+
+    // @brief Compute the average of array<double,3>.
+    //
+    // @param [in] vector of array<double,3>
+    // @return     array<double,3> (average of the entries of the given vector).
+    std::array<double,3> getAverageArr(const std::vector<std::array<double,3>>& vec) const;
+
+public:
+    /// @brief Refine a single cell and return a shared pointer of CpGridData type.
+    ///
+    /// refineSingleCell() takes a cell and refines it in a chosen amount of cells (per direction); creating the
+    /// geometries, topological relations, etc. Stored in a CpGridData object. Additionally, containers for
+    /// parent-to-new-born entities are buil, as well as, new-born-to-parent. Maps(<int,bool>) to detect parent
+    /// faces or cells are also provided. (Cell with 6 faces required).
+    ///
+    /// @param [in] cells_per_dim                 Number of (refined) cells in each direction that each parent cell should be refined to.
+    /// @param [in] parent_idx                    Parent cell index, cell to be refined.
+    ///
+    /// @return refined_grid_ptr                  Shared pointer pointing at refined_grid.
+    /// @return parent_to_refined_corners         For each corner of the parent cell, we store the index of the
+    ///                                           refined corner that coincides with the old one.
+    ///                                           We assume they are ordered 0,1,..7
+    ///                                                              6---7
+    ///                                                      2---3   |   | TOP FACE
+    ///                                                      |   |   4---5
+    ///                                                      0---1 BOTTOM FACE
+    /// @return parent_to_children_faces/cell     For each parent face/cell, we store its child-face/cell indices.
+    ///                                           {parent face/cell index in coarse level, {indices of its children in refined level}}
+    /// @return child_to_parent_faces/cells       {child index, parent index}
+    std::tuple< const std::shared_ptr<CpGridData>,
+                const std::vector<std::array<int,2>>,                // parent_to_refined_corners(~boundary_old_to_new_corners)
+                const std::vector<std::tuple<int,std::vector<int>>>, // parent_to_children_faces (~boundary_old_to_new_faces)
+                const std::tuple<int, std::vector<int>>,             // parent_to_children_cells
+                const std::vector<std::array<int,2>>,                // child_to_parent_faces
+                const std::vector<std::array<int,2>>>                // child_to_parent_cells
+    refineSingleCell(const std::array<int,3>& cells_per_dim, const int& parent_idx) const;
+
+    /// @brief Refine a (connected block-shaped) patch of cells. Based on the patch, a Geometry<3,3> object is created and refined.
+    ///
+    /// @param [in] cells_per_dim            Number of (refined) cells in each direction that each parent cell should be refined to.
+    /// @param [in] startIJK                 Cartesian triplet index where the patch starts.
+    /// @param [in] endIJK                   Cartesian triplet index where the patch ends.
+    ///                                      Last cell part of the lgr will be {endijk[0]-1, ... endIJK[2]-1}.
+    ///
+    /// @return refined_grid_ptr                   Shared pointer of CpGridData type, pointing at the refined_grid
+    /// @return boundary_old_to_new_corners/faces  Corner/face indices on the patch-boundary associated with new-born-entity indices.
+    /// @return parent_to_children_faces/cell      For each parent face/cell, we store its child-face/cell indices.
+    ///                                            {parent face/cell index in coarse level, {indices of its children in refined level}}
+    /// @return child_to_parent_faces/cells        {child index, parent index}
+    std::tuple< std::shared_ptr<CpGridData>,
+                const std::vector<std::array<int,2>>,                // boundary_old_to_new_corners
+                const std::vector<std::tuple<int,std::vector<int>>>, // boundary_old_to_new_faces
+                const std::vector<std::tuple<int,std::vector<int>>>, // parent_to_children_faces
+                const std::vector<std::tuple<int,std::vector<int>>>, // parent_to_children_cell
+                const std::vector<std::array<int,2>>,                // child_to_parent_faces
+                const std::vector<std::array<int,2>>>                // child_to_parent_cells
+    refinePatch(const std::array<int,3>& cells_per_dim, const std::array<int,3>& startIJK, const std::array<int,3>& endIJK) const;
+
+    // @breif Compute center of an entity/element/cell in the Eclipse way:
+    //        - Average of the 4 corners of the bottom face.
+    //        - Average of the 4 corners of the top face.
+    //        Return average of the previous computations.
+    // @param [in]   int   Index of a cell.
+    // @return            'eclipse centroid'
+    std::array<double,3> computeEclCentroid(const int idx) const;
+
+    // @breif Compute center of an entity/element/cell in the Eclipse way:
+    //        - Average of the 4 corners of the bottom face.
+    //        - Average of the 4 corners of the top face.
+    //        Return average of the previous computations.
+    // @param [in]   Entity<0>   Entity
+    // @return                   'eclipse centroid'
+    std::array<double,3> computeEclCentroid(const Entity<0>& elem) const;
+
     // Make unique boundary ids for all intersections.
     void computeUniqueBoundaryIds();
 
@@ -275,6 +510,13 @@ public:
     {
         return *index_set_;
     }
+
+    /// Get the local index set.
+    const cpgrid::IdSet& localIdSet() const
+    {
+        return *local_id_set_;
+    }
+
     /// The logical cartesian size of the grid.
     /// This function is not part of the Dune grid interface,
     /// and should be used with caution.
@@ -298,26 +540,29 @@ public:
     template<class DataHandle>
     void communicate(DataHandle& data, InterfaceType iftype, CommunicationDirection dir);
 
+    /// \brief The type of the mpi communicator.
+    using MPICommunicator = CpGridDataTraits::MPICommunicator ;
+    /// \brief The type of the collective communication.
+    using Communication = CpGridDataTraits::Communication;
+    using CollectiveCommunication = CpGridDataTraits::CollectiveCommunication;
 #if HAVE_MPI
-#if DUNE_VERSION_NEWER(DUNE_GRID, 2, 7)
+    /// \brief The type of the set of the attributes
+    using AttributeSet = CpGridDataTraits::AttributeSet;
+
     /// \brief The type of the  Communicator.
-    using Communicator = VariableSizeCommunicator<>;
-#else
-    /// \brief The type of the Communicator.
-    using Communicator = Opm::VariableSizeCommunicator<>;
-#endif
+    using Communicator = CpGridDataTraits::Communicator;
 
     /// \brief The type of the map describing communication interfaces.
-    using InterfaceMap = Communicator::InterfaceMap;
+    using InterfaceMap = CpGridDataTraits::InterfaceMap;
 
     /// \brief type of OwnerOverlap communication for cells
-    using CommunicationType = Dune::OwnerOverlapCopyCommunication<int,int>;
+    using CommunicationType = CpGridDataTraits::CommunicationType;
 
     /// \brief The type of the parallel index set
-    using  ParallelIndexSet = CommunicationType::ParallelIndexSet;
+    using  ParallelIndexSet = CpGridDataTraits::ParallelIndexSet;
 
     /// \brief The type of the remote indices information
-    using RemoteIndices = Dune::RemoteIndices<ParallelIndexSet>;
+    using RemoteIndices = CpGridDataTraits::RemoteIndices;
 
     /// \brief Get the owner-overlap-copy communication for cells
     ///
@@ -356,14 +601,6 @@ public:
     }
 #endif
 
-#ifdef HAVE_DUNE_ISTL
-    /// \brief The type of the set of the attributes
-    typedef Dune::OwnerOverlapCopyAttributeSet::AttributeSet AttributeSet;
-#else
-    /// \brief The type of the set of the attributes
-    enum AttributeSet{owner, overlap, copy};
-#endif
-
     /// \brief Get sorted active cell indices of numerical aquifer
     const std::vector<int>& sortedNumAquiferCells() const
     {
@@ -484,7 +721,7 @@ private:
      */
     std::vector<int>                  global_cell_;
     /** @brief The tag of the faces. */
-    cpgrid::EntityVariable<enum face_tag, 1> face_tag_; // {LEFT, BACK, TOP}
+    cpgrid::EntityVariable<enum face_tag, 1> face_tag_;
     /** @brief The geometries representing the grid. */
     cpgrid::DefaultGeometryPolicy geometry_;
     /** @brief The type of a point in the grid. */
@@ -494,22 +731,32 @@ private:
     /** @brief The boundary ids. */
     cpgrid::EntityVariable<int, 1> unique_boundary_ids_;
     /** @brief The index set of the grid (level). */
-    cpgrid::IndexSet* index_set_;
+    std::unique_ptr<cpgrid::IndexSet> index_set_;
     /** @brief The internal local id set (not exported). */
-    const cpgrid::IdSet* local_id_set_;
+    std::shared_ptr<const cpgrid::IdSet> local_id_set_;
     /** @brief The global id set (used also as local id set). */
-    LevelGlobalIdSet* global_id_set_;
+    std::shared_ptr<LevelGlobalIdSet> global_id_set_;
     /** @brief The indicator of the partition type of the entities */
-    PartitionTypeIndicator* partition_type_indicator_;
+    std::shared_ptr<PartitionTypeIndicator> partition_type_indicator_;
+    /** Level of the current CpGridData (0 when it's "GLOBAL", 1,2,.. for LGRs). */
+    int level_{0};
+    /** Copy of (CpGrid object).data_ associated with the CpGridData object. */
+    std::vector<std::shared_ptr<CpGridData>>* level_data_ptr_;
+    // SUITABLE FOR ALL LEVELS EXCEPT FOR LEAFVIEW
+    /** Map between level and leafview cell indices. Only cells (from that level) that appear in leafview count. */  
+    std::vector<int> level_to_leaf_cells_; // In entry 'level cell index', we store 'leafview cell index'
+    /** Parent cells and their children. Entry is {-1, {}} when cell has no children.*/ // {level LGR, {child0, child1, ...}}
+    std::vector<std::tuple<int,std::vector<int>>> parent_to_children_cells_; 
+    /** Amount of children cells per parent cell in each direction. */ // {# children in x-direction, ... y-, ... z-}
+    std::array<int,3> cells_per_dim_;
+    // SUITABLE ONLY FOR LEAFVIEW
+    /** Relation between leafview and (possible different) level(s) cell indices. */ // {level, cell index in that level}
+    std::vector<std::array<int,2>> leaf_to_level_cells_;
+    // SUITABLE FOR ALL LEVELS INCLUDING LEAFVIEW
+    /** Child cells and their parents. Entry is {-1,-1} when cell has no father. */ // {level parent cell, parent cell index}
+    std::vector<std::array<int,2>> child_to_parent_cells_;
+
 
-    /// \brief The type of the collective communication.
-    typedef MPIHelper::MPICommunicator MPICommunicator;
-    #if DUNE_VERSION_NEWER(DUNE_GRID, 2, 7)
-    using Communication = Dune::Communication<MPICommunicator>;
-#else
-    using CollectiveCommunication = Dune::CollectiveCommunication<MPICommunicator>;
-    using Communication = Dune::CollectiveCommunication<MPICommunicator>;
-#endif
     /// \brief Object for collective communication operations.
     Communication ccobj_;
 
@@ -560,6 +807,8 @@ private:
     friend class PartitionTypeIndicator;
 };
 
+
+
 #if HAVE_MPI
 
 namespace
