@@ -153,11 +153,12 @@ BOOST_AUTO_TEST_CASE(cellgeom)
 //     for (int i = 0; i < 8; ++i) {
 //         std::cout << corners[i] << std::endl;
 //     }
-    cpgrid::EntityVariable<cpgrid::Geometry<0, 3>, 3> pg;
-    pg.reserve(8);
+    // call empty constructor initialze an object that we point to.
+    auto pg = std::make_shared<cpgrid::EntityVariable<cpgrid::Geometry<0, 3>, 3>>();
+    (*pg).reserve(8);
     for (const auto& crn : corners)
     {
-        pg.push_back(cpgrid::Geometry<0, 3>(crn));
+        (*pg).push_back(cpgrid::Geometry<0, 3>(crn));
     }
 
     int cor_idx[8] = { 0, 1, 2, 3, 4, 5, 6, 7 };
@@ -208,11 +209,12 @@ BOOST_AUTO_TEST_CASE(cellgeom)
     corners[5][2] = 0.0;
     corners[7][2] = 0.0;
 
-    cpgrid::EntityVariable<cpgrid::Geometry<0, 3>, 3> pg1;
-    pg1.reserve(8);
+    // call empty constructor initialze an object that we point to.
+    auto pg1 = std::make_shared<cpgrid::EntityVariable<cpgrid::Geometry<0, 3>, 3>>();
+    (*pg1).reserve(8);
     for (const auto& crn : corners)
     {
-        pg1.push_back(cpgrid::Geometry<0, 3>(crn));
+        (*pg1).push_back(cpgrid::Geometry<0, 3>(crn));
     }
     g = Geometry(c, v, pg1, cor_idx);
 
@@ -410,9 +412,10 @@ void refine_and_check(const cpgrid::Geometry<3, 3>& parent_geometry,
     cpgrid::EntityVariable<enum face_tag, 1>& face_tags = child_view_data.face_tag_;
     cpgrid::SignedEntityVariable<Dune::FieldVector<double,3>, 1>& face_normals = child_view_data.face_normals_;
 
-    parent_geometry.refine(cells, geometries, cell_to_point,
-                           cell_to_face, face_to_point, face_to_cell,
-                           face_tags, face_normals);
+    parent_geometry.refineCellifiedPatch(cells, geometries, cell_to_point,
+                                         cell_to_face, face_to_point, face_to_cell,
+                                         face_tags, face_normals,
+                                         {1,1,1}, {1.}, {1.}, {1.});
     check_refined_grid(parent_geometry, geometries.template geomVector<0>(),
                        geometries.template geomVector<1>(),
                        geometries.template geomVector<3>(), cells);
@@ -482,7 +485,7 @@ void refine_and_check(const cpgrid::Geometry<3, 3>& parent_geometry,
 
                         CHECK_COORDINATES(intersection_match.centerUnitOuterNormal(), intersection.centerUnitOuterNormal());
                         const auto& geom_match = intersection_match.geometry();
-                        BOOST_TEST(0 == 1e-11, boost::test_tools::tolerance(1e-8));
+                        BOOST_TEST(0.0 == 1e-11, boost::test_tools::tolerance(1e-8));
                         const auto& geom =  intersection.geometry();
                         BOOST_CHECK_CLOSE(geom_match.volume(), geom.volume(), 1e-6);
                         CHECK_COORDINATES(geom_match.center(), geom.center());
@@ -525,9 +528,10 @@ BOOST_AUTO_TEST_CASE(refine_simple_cube)
     const GlobalCoordinate c = {0.5, 0.5, 0.5};
     const Geometry::ctype v = 1.0;
 
-    cpgrid::EntityVariable<cpgrid::Geometry<0, 3>, 3> pg;
+    // call empty constructor initialze an object that we point to.
+    auto pg = std::make_shared<cpgrid::EntityVariable<cpgrid::Geometry<0, 3>, 3>>();
     for (const auto& crn : corners) {
-        pg.push_back(cpgrid::Geometry<0, 3>(crn));
+        (*pg).push_back(cpgrid::Geometry<0, 3>(crn));
     }
 
     int cor_idx[8] = {0, 1, 2, 3, 4, 5, 6, 7};
@@ -566,9 +570,10 @@ BOOST_AUTO_TEST_CASE(refine_distorted_cube)
         }
     }
 
-    cpgrid::EntityVariable<cpgrid::Geometry<0, 3>, 3> pg;
+    // call empty constructor initialze an object that we point to.
+    auto pg = std::make_shared<cpgrid::EntityVariable<cpgrid::Geometry<0, 3>, 3>>();
     for (const auto& crn : corners) {
-        pg.push_back(cpgrid::Geometry<0, 3>(crn));
+        (*pg).push_back(cpgrid::Geometry<0, 3>(crn));
     }
 
     int cor_idx[8] = {0, 1, 2, 3, 4, 5, 6, 7};
